@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2011 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2012 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ using System;
 using System.Collections.Generic;
 using System.Text;
 using System.Xml.Serialization;
+using System.ComponentModel;
 
 using KeePass.Ecas;
 
@@ -35,6 +36,7 @@ namespace KeePass.App.Configuration
 		}
 
 		private string m_strLanguageFile = string.Empty; // = English
+		[DefaultValue("")]
 		public string LanguageFile
 		{
 			get { return m_strLanguageFile; }
@@ -46,12 +48,26 @@ namespace KeePass.App.Configuration
 		}
 
 		private bool m_bHelpUseLocal = false;
+		[DefaultValue(false)]
 		public bool HelpUseLocal
 		{
 			get { return m_bHelpUseLocal; }
 			set { m_bHelpUseLocal = value; }
 		}
 
+		// Serialize DateTime with TimeUtil
+		private string m_strLastUpdChk = string.Empty;
+		[DefaultValue("")]
+		public string LastUpdateCheck
+		{
+			get { return m_strLastUpdChk; }
+			set
+			{
+				if(value == null) throw new ArgumentNullException("value");
+				m_strLastUpdChk = value;
+			}
+		}
+
 		private IOConnectionInfo m_ioLastDb = new IOConnectionInfo();
 		public IOConnectionInfo LastUsedFile
 		{
@@ -97,6 +113,7 @@ namespace KeePass.App.Configuration
 		}
 
 		private bool m_bVerifyFile = true;
+		[DefaultValue(true)]
 		public bool VerifyWrittenFileAfterSaving
 		{
 			get { return m_bVerifyFile; }
@@ -104,12 +121,21 @@ namespace KeePass.App.Configuration
 		}
 
 		private bool m_bTransactedWrites = true;
+		[DefaultValue(true)]
 		public bool UseTransactedFileWrites
 		{
 			get { return m_bTransactedWrites; }
 			set { m_bTransactedWrites = value; }
 		}
 
+		private bool m_bFileLocks = false;
+		[DefaultValue(false)]
+		public bool UseFileLocks
+		{
+			get { return m_bFileLocks; }
+			set { m_bFileLocks = value; }
+		}
+
 		private AceCloseDb m_fc = new AceCloseDb();
 		public AceCloseDb FileClosing
 		{
@@ -133,6 +159,7 @@ namespace KeePass.App.Configuration
 		}
 
 		private string m_strPluginCachePath = string.Empty;
+		[DefaultValue("")]
 		public string PluginCachePath
 		{
 			get { return m_strPluginCachePath; }
@@ -151,6 +178,7 @@ namespace KeePass.App.Configuration
 		}
 
 		private bool m_bOpenLastDb = true;
+		[DefaultValue(true)]
 		public bool OpenLastFile
 		{
 			get { return m_bOpenLastDb; }
@@ -158,13 +186,23 @@ namespace KeePass.App.Configuration
 		}
 
 		private bool m_bCheckForUpdate = false;
+		// [DefaultValue(false)] // Avoid user confusion with 'Configured' setting
 		public bool CheckForUpdate
 		{
 			get { return m_bCheckForUpdate; }
 			set { m_bCheckForUpdate = value; }
 		}
 
+		private bool m_bCheckForUpdateCfg = false;
+		[DefaultValue(false)]
+		public bool CheckForUpdateConfigured
+		{
+			get { return m_bCheckForUpdateCfg; }
+			set { m_bCheckForUpdateCfg = value; }
+		}
+
 		private bool m_bMinimizedAndLocked = false;
+		[DefaultValue(false)]
 		public bool MinimizedAndLocked
 		{
 			get { return m_bMinimizedAndLocked; }
@@ -172,6 +210,7 @@ namespace KeePass.App.Configuration
 		}
 
 		private bool m_bPlgDeleteOld = true;
+		[DefaultValue(true)]
 		public bool PluginCacheDeleteOld
 		{
 			get { return m_bPlgDeleteOld; }
@@ -179,6 +218,7 @@ namespace KeePass.App.Configuration
 		}
 
 		private bool m_bClearPlgCache = false;
+		[DefaultValue(false)]
 		public bool PluginCacheClearOnce
 		{
 			get { return m_bClearPlgCache; }
@@ -193,6 +233,7 @@ namespace KeePass.App.Configuration
 		}
 
 		private bool m_bShowExpiredEntries = false;
+		[DefaultValue(false)]
 		public bool ShowExpiredEntries
 		{
 			get { return m_bShowExpiredEntries; }
@@ -200,6 +241,7 @@ namespace KeePass.App.Configuration
 		}
 
 		private bool m_bShowSoonToExpireEntries = false;
+		[DefaultValue(false)]
 		public bool ShowSoonToExpireEntries
 		{
 			get { return m_bShowSoonToExpireEntries; }
@@ -214,6 +256,7 @@ namespace KeePass.App.Configuration
 		}
 
 		private bool m_bAutoSave = false;
+		[DefaultValue(false)]
 		public bool AutoSave
 		{
 			get { return m_bAutoSave; }
