@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2011 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2012 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ using System;
 using System.Collections.Generic;
 using System.Text;
 using System.Drawing;
+using System.ComponentModel;
 using System.Diagnostics;
 
 using KeePass.UI;
@@ -55,7 +56,23 @@ namespace KeePass.App.Configuration
 		None = 0,
 		DisableOptions = 0x1,
 		DisablePlugins = 0x2,
-		DisableTriggers = 0x4
+		DisableTriggers = 0x4,
+		DisableKeyChangeDays = 0x8
+	}
+
+	[Flags]
+	public enum AceAutoTypeCtxFlags : long
+	{
+		None = 0,
+
+		ColTitle = 0x1,
+		ColUserName = 0x2,
+		ColPassword = 0x4,
+		ColUrl = 0x8,
+		ColNotes = 0x10,
+		ColSequence = 0x20,
+
+		Default = (ColTitle | ColUserName | ColUrl | ColSequence)
 	}
 
 	public sealed class AceUI
@@ -109,6 +126,7 @@ namespace KeePass.App.Configuration
 		}
 
 		private bool m_bForceSysFont = true;
+		[DefaultValue(true)]
 		public bool ForceSystemFontUnix
 		{
 			get { return m_bForceSysFont; }
@@ -123,6 +141,7 @@ namespace KeePass.App.Configuration
 		}
 
 		private bool m_bShowImportStatusDlg = true;
+		[DefaultValue(true)]
 		public bool ShowImportStatusDialog
 		{
 			get { return m_bShowImportStatusDlg; }
@@ -130,6 +149,7 @@ namespace KeePass.App.Configuration
 		}
 
 		private bool m_bShowDbMntncResDlg = true;
+		[DefaultValue(true)]
 		public bool ShowDbMntncResultsDialog
 		{
 			get { return m_bShowDbMntncResDlg; }
@@ -137,6 +157,7 @@ namespace KeePass.App.Configuration
 		}
 
 		private bool m_bUseCustomTsRenderer = true;
+		[DefaultValue(true)]
 		public bool UseCustomToolStripRenderer
 		{
 			get { return m_bUseCustomTsRenderer; }
@@ -144,6 +165,7 @@ namespace KeePass.App.Configuration
 		}
 
 		private bool m_bOptScreenReader = false;
+		[DefaultValue(false)]
 		public bool OptimizeForScreenReader
 		{
 			get { return m_bOptScreenReader; }
@@ -151,6 +173,7 @@ namespace KeePass.App.Configuration
 		}
 
 		private string m_strDataEditorRect = string.Empty;
+		[DefaultValue("")]
 		public string DataEditorRect
 		{
 			get { return m_strDataEditorRect; }
@@ -173,6 +196,7 @@ namespace KeePass.App.Configuration
 		}
 
 		private bool m_bDeWordWrap = true;
+		[DefaultValue(true)]
 		public bool DataEditorWordWrap
 		{
 			get { return m_bDeWordWrap; }
@@ -180,6 +204,7 @@ namespace KeePass.App.Configuration
 		}
 
 		private string m_strCharPickerRect = string.Empty;
+		[DefaultValue("")]
 		public string CharPickerRect
 		{
 			get { return m_strCharPickerRect; }
@@ -190,6 +215,38 @@ namespace KeePass.App.Configuration
 			}
 		}
 
+		private string m_strAutoTypeCtxRect = string.Empty;
+		[DefaultValue("")]
+		public string AutoTypeCtxRect
+		{
+			get { return m_strAutoTypeCtxRect; }
+			set
+			{
+				if(value == null) throw new ArgumentNullException("value");
+				m_strAutoTypeCtxRect = value;
+			}
+		}
+
+		private long m_lAutoTypeCtxFlags = (long)AceAutoTypeCtxFlags.Default;
+		[DefaultValue((long)AceAutoTypeCtxFlags.Default)]
+		public long AutoTypeCtxFlags
+		{
+			get { return m_lAutoTypeCtxFlags; }
+			set { m_lAutoTypeCtxFlags = value; }
+		}
+
+		private string m_strAutoTypeCtxColWidths = string.Empty;
+		[DefaultValue("")]
+		public string AutoTypeCtxColumnWidths
+		{
+			get { return m_strAutoTypeCtxColWidths; }
+			set
+			{
+				if(value == null) throw new ArgumentNullException("value");
+				m_strAutoTypeCtxColWidths = value;
+			}
+		}
+
 		private ulong m_uUIFlags = (ulong)AceUIFlags.None;
 		public ulong UIFlags
 		{
@@ -219,6 +276,7 @@ namespace KeePass.App.Configuration
 		// }
 
 		private bool m_bSecDeskSound = true;
+		[DefaultValue(true)]
 		public bool SecureDesktopPlaySound
 		{
 			get { return m_bSecDeskSound; }
@@ -233,6 +291,7 @@ namespace KeePass.App.Configuration
 		}
 
 		private bool m_bSepHiding = false;
+		[DefaultValue(false)]
 		public bool SeparateHidingSettings
 		{
 			get { return m_bSepHiding; }
@@ -240,6 +299,7 @@ namespace KeePass.App.Configuration
 		}
 
 		private bool m_bHideInEntryDialog = true;
+		[DefaultValue(true)]
 		public bool HideInEntryWindow
 		{
 			get { return m_bHideInEntryDialog; }
@@ -247,6 +307,7 @@ namespace KeePass.App.Configuration
 		}
 
 		private bool m_bUnhideBtnAlsoUnhidesSec = false;
+		[DefaultValue(false)]
 		public bool UnhideButtonAlsoUnhidesSource
 		{
 			get { return m_bUnhideBtnAlsoUnhidesSec; }
