@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2011 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2012 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -62,7 +62,7 @@ namespace KeePass.Forms
 				strTitle, strDesc);
 			this.Icon = Properties.Resources.KeePass;
 
-			m_lvComponents.Columns.Add(KPRes.Components, 100, HorizontalAlignment.Left);
+			m_lvComponents.Columns.Add(KPRes.Component, 100, HorizontalAlignment.Left);
 			m_lvComponents.Columns.Add(KPRes.Version, 100, HorizontalAlignment.Left);
 
 			try { GetAppComponents(); }
