@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2011 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2012 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -130,7 +130,7 @@ namespace KeePass.Forms
 			ListView.SelectedListViewItemCollection slvic = m_lvEntries.SelectedItems;
 			if(slvic.Count == 1)
 			{
-				m_peSelected = slvic[0].Tag as PwEntry;
+				m_peSelected = (slvic[0].Tag as PwEntry);
 
 				if(bSetDialogResult) this.DialogResult = DialogResult.OK;
 				return true;
