@@ -1,6 +1,6 @@
 ﻿/*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2011 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2012 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -71,7 +71,6 @@ namespace KeePass.Forms
 			this.Icon = Properties.Resources.KeePass;
 
 			UIUtil.SetExplorerTheme(m_lvEntries.Handle);
-
 			if(UISystemFonts.ListFont != null)
 				m_lvEntries.Font = UISystemFonts.ListFont;
 
@@ -157,6 +156,7 @@ namespace KeePass.Forms
 
 			SearchParameters sp = SearchParameters.None;
 			sp.SearchString = strSearch;
+			sp.RespectEntrySearchingDisabled = false;
 
 			if(tchField == 'T') sp.SearchInTitles = true;
 			else if(tchField == 'U') sp.SearchInUserNames = true;
@@ -167,7 +167,7 @@ namespace KeePass.Forms
 			else { Debug.Assert(false); return true; }
 
 			PwObjectList<PwEntry> l = new PwObjectList<PwEntry>();
-			m_pgEntrySource.SearchEntries(sp, l, false);
+			m_pgEntrySource.SearchEntries(sp, l);
 
 			if(l.UCount == 0) { Debug.Assert(false); return false; }
 			else if(l.UCount == 1) return false;
@@ -177,7 +177,7 @@ namespace KeePass.Forms
 
 		private void OnBtnOK(object sender, EventArgs e)
 		{
-			if(!this.CreateResultRef()) this.DialogResult = DialogResult.None;
+			if(!CreateResultRef()) this.DialogResult = DialogResult.None;
 		}
 
 		private void OnBtnCancel(object sender, EventArgs e)
@@ -218,7 +218,7 @@ namespace KeePass.Forms
 				sp.SearchInPasswords = true;
 
 				PwObjectList<PwEntry> lResults = new PwObjectList<PwEntry>();
-				m_pgEntrySource.SearchEntries(sp, lResults, true);
+				m_pgEntrySource.SearchEntries(sp, lResults);
 
 				UIUtil.CreateEntryList(m_lvEntries, lResults, m_vColumns, m_ilIcons);
 			}
