@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2011 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2012 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -253,15 +253,16 @@ namespace KeePass.Forms
 				KeyProvider kp = Program.KeyProviderPool.Get(strKeyFile);
 				if((kp != null) && m_bSecureDesktop)
 				{
-					if(kp.GetKeyMightShowGui)
+					if(!kp.SecureDesktopCompatible)
 					{
-						MessageService.ShowWarning(KPRes.KeyProvWithGuiOnSD,
-							KPRes.KeyProvWithGuiOnSDHint);
+						MessageService.ShowWarning(KPRes.KeyProvIncmpWithSD,
+							KPRes.KeyProvIncmpWithSDHint);
 						return false;
 					}
 				}
 
-				KeyProviderQueryContext ctxKP = new KeyProviderQueryContext(m_ioInfo, false);
+				KeyProviderQueryContext ctxKP = new KeyProviderQueryContext(
+					m_ioInfo, false, m_bSecureDesktop);
 
 				bool bPerformHash;
 				byte[] pbProvKey = Program.KeyProviderPool.GetKey(strKeyFile, ctxKP,
@@ -385,6 +386,8 @@ namespace KeePass.Forms
 			}
 
 			m_secPassword.EnableProtection(bHide);
+
+			if(!m_bInitializing) UIUtil.SetFocus(m_tbPassword, this);
 		}
 
 		private void OnBtnOK(object sender, EventArgs e)
