@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2011 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2012 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -67,7 +67,6 @@ namespace KeePass.Forms
 		private bool m_bRestart = false;
 		private ListSorter m_pListSorter = new ListSorter();
 		private ListViewSortMenu m_lvsmMenu = null;
-		private bool m_bBlockQuickFind = false;
 
 		private bool m_bDraggingEntries = false;
 
@@ -81,6 +80,18 @@ namespace KeePass.Forms
 		/// </summary>
 		public MainForm()
 		{
+			try
+			{
+				m_nTaskbarButtonMessage = NativeMethods.RegisterWindowMessage(
+					"TaskbarButtonCreated");
+				m_bTaskbarButtonMessage = (m_nTaskbarButtonMessage != 0);
+			}
+			catch(Exception)
+			{
+				m_nTaskbarButtonMessage = 0x1E8F46A7; // Unlikely to occur
+				m_bTaskbarButtonMessage = false;
+			}
+
 			string strIso6391 = Program.Translation.Properties.Iso6391Code;
 			if(strIso6391.Length > 0)
 			{
@@ -104,6 +115,8 @@ namespace KeePass.Forms
 			Program.Translation.ApplyTo("KeePass.Forms.MainForm.m_ctxGroupList", m_ctxGroupList.Items);
 			Program.Translation.ApplyTo("KeePass.Forms.MainForm.m_ctxTray", m_ctxTray.Items);
 
+			m_asyncListUpdate = new AsyncPwListUpdate(m_lvEntries);
+
 			m_splitHorizontal.InitEx(this.Controls, m_menuMain);
 			m_splitVertical.InitEx(this.Controls, m_menuMain);
 
@@ -113,7 +126,8 @@ namespace KeePass.Forms
 			{
 				// Workaround for tab bar height bug in Mono
 				// https://sourceforge.net/projects/keepass/forums/forum/329221/topic/4519750
-				m_tabMain.Height += 3;
+				// https://bugs.launchpad.net/ubuntu/+source/keepass2/+bug/891029
+				m_tabMain.Height += 5;
 			}
 		}
 
@@ -133,18 +147,22 @@ namespace KeePass.Forms
 			// m_imgFileSaveAllDisabled = Properties.Resources.B16x16_File_SaveAll_Disabled;
 			m_ilCurrentIcons = m_ilClientIcons;
 
+			m_ctxEntryOpenUrl.Text = KPRes.OpenCmd;
+
 			m_ntfTray = new NotifyIconEx(this.components);
 			m_ntfTray.ContextMenuStrip = m_ctxTray;
 			m_ntfTray.Visible = true;
 			m_ntfTray.SetHandlers(OnSystemTrayClick, OnSystemTrayDoubleClick,
 				OnSystemTrayMouseDown);
 
+			m_ctxTrayTray.Font = FontUtil.CreateFont(m_ctxTrayTray.Font, FontStyle.Bold);
+
 			m_nLockTimerMax = (int)Program.Config.Security.WorkspaceLocking.LockAfterTime;
 			m_nClipClearMax = Program.Config.Security.ClipboardClearAfterSeconds;
 
 			NativeLib.AllowNative = Program.Config.Native.NativeKeyTransformations;
 
-			m_ctxEntryPreviewContextMenu.Attach(m_richEntryView);
+			m_ctxEntryPreviewContextMenu.Attach(m_richEntryView, this);
 
 			m_dynCustomStrings = new DynamicMenu(m_ctxEntryCopyCustomString.DropDownItems);
 			m_dynCustomStrings.MenuClick += this.OnCopyCustomString;
@@ -164,6 +182,8 @@ namespace KeePass.Forms
 			m_dynRemoveTag = new DynamicMenu(m_ctxEntrySelectedRemoveTag.DropDownItems);
 			m_dynRemoveTag.MenuClick += this.OnRemoveEntryTag;
 
+			m_dynOpenUrl = new OpenWithMenu(m_ctxEntryUrl);
+
 			EntryTemplates.Init(m_tbAddEntry);
 
 			m_menuEdit.DropDownItems.Insert(0, new ToolStripSeparator());
@@ -172,29 +192,33 @@ namespace KeePass.Forms
 			InsertToolStripItem(m_menuEdit, m_ctxEntryDelete, new EventHandler(OnEntryDelete), true);
 			InsertToolStripItem(m_menuEdit, m_ctxEntryDuplicate, new EventHandler(OnEntryDuplicate), true);
 			InsertToolStripItem(m_menuEdit, m_ctxEntryEdit, new EventHandler(OnEntryEdit), true);
-			InsertToolStripItem(m_menuEdit, m_ctxEntryAdd, new EventHandler(OnEntryAdd), true);
+			ToolStripMenuItem tsmiAddEntry = InsertToolStripItem(m_menuEdit,
+				m_ctxEntryAdd, new EventHandler(OnEntryAdd), true);
 			m_menuEdit.DropDownItems.Insert(0, new ToolStripSeparator());
 			InsertToolStripItem(m_menuEdit, m_ctxGroupDelete, new EventHandler(OnGroupsDelete), true);
 			InsertToolStripItem(m_menuEdit, m_ctxGroupEdit, new EventHandler(OnGroupsEdit), true);
 			InsertToolStripItem(m_menuEdit, m_ctxGroupAdd, new EventHandler(OnGroupsAdd), true);
 
-			UIUtil.ConfigureTbButton(m_tbNewDatabase, KPRes.ToolBarNew, null);
-			UIUtil.ConfigureTbButton(m_tbOpenDatabase, KPRes.ToolBarOpen, null);
-			UIUtil.ConfigureTbButton(m_tbSaveDatabase, KPRes.Save, null);
-			UIUtil.ConfigureTbButton(m_tbSaveAll, KPRes.ToolBarSaveAll, null);
-			UIUtil.ConfigureTbButton(m_tbAddEntry, KPRes.AddEntry, null);
-			UIUtil.ConfigureTbButton(m_tbAddEntryDefault, KPRes.AddEntryBtn, null);
-			UIUtil.ConfigureTbButton(m_tbCopyUserName, KPRes.CopyUserFull, null);
-			UIUtil.ConfigureTbButton(m_tbCopyPassword, KPRes.CopyPasswordFull, null);
-			UIUtil.ConfigureTbButton(m_tbFind, KPRes.Find + "...", null);
-			UIUtil.ConfigureTbButton(m_tbEntryViewsDropDown, null, KPRes.ShowEntries);
-			UIUtil.ConfigureTbButton(m_tbViewsShowAll, KPRes.ShowAllEntries, null);
-			UIUtil.ConfigureTbButton(m_tbViewsShowExpired, KPRes.ShowExpiredEntries, null);
-			UIUtil.ConfigureTbButton(m_tbLockWorkspace, KPRes.LockMenuLock, null);
+			UIUtil.AssignShortcut(tsmiAddEntry, Keys.Control | Keys.I);
+
+			UIUtil.ConfigureTbButton(m_tbNewDatabase, KPRes.ToolBarNew, null, m_menuFileNew);
+			UIUtil.ConfigureTbButton(m_tbOpenDatabase, KPRes.ToolBarOpen, null, m_menuFileOpenLocal);
+			UIUtil.ConfigureTbButton(m_tbSaveDatabase, KPRes.Save, null, m_menuFileSave);
+			UIUtil.ConfigureTbButton(m_tbSaveAll, KPRes.ToolBarSaveAll, null, null);
+			UIUtil.ConfigureTbButton(m_tbAddEntry, KPRes.AddEntry, null, null);
+			UIUtil.ConfigureTbButton(m_tbCopyUserName, KPRes.CopyUserFull, null, m_ctxEntryCopyUserName);
+			UIUtil.ConfigureTbButton(m_tbCopyPassword, KPRes.CopyPasswordFull, null, m_ctxEntryCopyPassword);
+			UIUtil.ConfigureTbButton(m_tbFind, KPRes.Find + "...", null, m_menuEditFind);
+			UIUtil.ConfigureTbButton(m_tbEntryViewsDropDown, null, KPRes.ShowEntries, null);
+			UIUtil.ConfigureTbButton(m_tbLockWorkspace, KPRes.LockMenuLock, null, m_menuFileLock);
 			UIUtil.ConfigureTbButton(m_tbQuickFind, null, KPRes.SearchQuickPrompt +
-				" (" + KPRes.KeyboardKeyCtrl + "+E)");
+				" (" + KPRes.KeyboardKeyCtrl + "+E)", null);
 			UIUtil.ConfigureTbButton(m_tbCloseTab, StrUtil.RemoveAccelerator(
-				KPRes.CloseButton), null);
+				KPRes.CloseButton), null, m_menuFileClose);
+
+			CopyMenuItemText(m_tbAddEntryDefault, m_ctxEntryAdd);
+			CopyMenuItemText(m_tbViewsShowAll, m_menuEditShowAllEntries);
+			CopyMenuItemText(m_tbViewsShowExpired, m_menuEditShowExpired);
 
 			UIUtil.EnableAutoCompletion(m_tbQuickFind, false);
 
@@ -283,6 +307,8 @@ namespace KeePass.Forms
 				UIUtil.SetExplorerTheme(m_lvEntries.Handle);
 			}
 
+			// m_tvGroups.QueryToolTip = UIUtil.GetPwGroupToolTipTN;
+
 			m_clrAlternateItemBgColor = UIUtil.GetAlternateColor(m_lvEntries.BackColor);
 
 			m_statusPartProgress.Visible = false;
@@ -290,7 +316,7 @@ namespace KeePass.Forms
 			if(bMaximizedSetting)
 			{
 				if((this.WindowState == FormWindowState.Normal) && !IsTrayed())
-					this.WindowState = FormWindowState.Maximized;
+					UIUtil.SetWindowState(this, FormWindowState.Maximized);
 			}
 
 			try
@@ -331,6 +357,8 @@ namespace KeePass.Forms
 				m_pluginManager.LoadAllPlugins(UrlUtil.GetFileDirectory(
 					WinUtil.GetExecutable(), false, true));
 
+			// Delete old files *after* loading plugins (when timestamps
+			// of loaded plugins have been updated already)
 			if(Program.Config.Application.Start.PluginCacheDeleteOld)
 				PlgxCache.DeleteOldFilesAsync();
 
@@ -358,6 +386,8 @@ namespace KeePass.Forms
 			UpdateUIState(false);
 			ApplyUICustomizations();
 
+			ThreadPool.QueueUserWorkItem(new WaitCallback(OnFormLoadParallelAsync));
+
 			Program.TriggerSystem.RaiseEvent(EcasEventIDs.AppInitPost);
 
 			if(Program.CommandLineArgs.FileName != null)
@@ -370,8 +400,10 @@ namespace KeePass.Forms
 					OpenDatabase(ioLastFile, null, false);
 			}
 
+			UpdateCheckEx.EnsureConfigured(this);
 			if(Program.Config.Application.Start.CheckForUpdate)
-				CheckForUpdate.StartAsync(PwDefs.VersionUrl, m_statusPartInfo);
+				UpdateCheckEx.Run(false, null);
+			// UpdateCheck.StartAsync(PwDefs.VersionUrl, m_statusPartInfo);
 
 			ResetDefaultFocus(null);
 
@@ -472,7 +504,8 @@ namespace KeePass.Forms
 				KPRes.Password));
 			pe.Strings.Set(PwDefs.NotesField, new ProtectedString(pd.MemoryProtection.ProtectNotes,
 				KPRes.Notes));
-			pe.AutoType.Set(KPRes.TargetWindow, @"{USERNAME}{TAB}{PASSWORD}{TAB}{ENTER}");
+			pe.AutoType.Add(new AutoTypeAssociation(KPRes.TargetWindow,
+				@"{USERNAME}{TAB}{PASSWORD}{TAB}{ENTER}"));
 			pd.RootGroup.AddEntry(pe, true);
 
 			pe = new PwEntry(true, true);
@@ -484,7 +517,7 @@ namespace KeePass.Forms
 				@"http://keepass.info/help/kb/kb090406_testform.html"));
 			pe.Strings.Set(PwDefs.PasswordField, new ProtectedString(pd.MemoryProtection.ProtectPassword,
 				"12345"));
-			pe.AutoType.Set("Test Form - KeePass*", string.Empty);
+			pe.AutoType.Add(new AutoTypeAssociation("Test Form - KeePass*", string.Empty));
 			pd.RootGroup.AddEntry(pe, true);
 
 #if DEBUG
@@ -661,6 +694,7 @@ namespace KeePass.Forms
 		private void OnFileExit(object sender, EventArgs e)
 		{
 			NotifyUserActivity();
+			if(UIIsInteractionBlocked()) { Debug.Assert(false); return; }
 
 			if(GlobalWindowManager.CanCloseAllWindows)
 				GlobalWindowManager.CloseAllWindows();
@@ -686,7 +720,8 @@ namespace KeePass.Forms
 
 		private void OnHelpCheckForUpdate(object sender, EventArgs e)
 		{
-			CheckForUpdate.StartAsync(PwDefs.VersionUrl, null);
+			// UpdateCheck.StartAsync(PwDefs.VersionUrl, null);
+			UpdateCheckEx.Run(true, this);
 		}
 
 		private void OnHelpAbout(object sender, EventArgs e)
@@ -701,7 +736,7 @@ namespace KeePass.Forms
 			Debug.Assert(pe != null); if(pe == null) return;
 
 			if(ClipboardUtil.CopyAndMinimize(pe.Strings.GetSafe(PwDefs.UserNameField),
-				true, this, pe, m_docMgr.ActiveDatabase))
+				true, this, pe, m_docMgr.SafeFindContainerOf(pe)))
 				StartClipboardCountdown();
 		}
 
@@ -717,7 +752,7 @@ namespace KeePass.Forms
 			}
 
 			if(ClipboardUtil.CopyAndMinimize(pe.Strings.GetSafe(PwDefs.PasswordField),
-				true, this, pe, m_docMgr.ActiveDatabase))
+				true, this, pe, m_docMgr.SafeFindContainerOf(pe)))
 				StartClipboardCountdown();
 		}
 
@@ -745,11 +780,12 @@ namespace KeePass.Forms
 			PwEntry pe = GetSelectedEntry(false);
 			if(pe != null)
 			{
-				try { AutoType.PerformIntoPreviousWindow(this, pe); }
-				catch(Exception ex)
+				try
 				{
-					MessageService.ShowWarning(ex);
+					AutoType.PerformIntoPreviousWindow(this, pe,
+						m_docMgr.SafeFindContainerOf(pe));
 				}
+				catch(Exception ex) { MessageService.ShowWarning(ex); }
 			}
 		}
 
@@ -768,13 +804,12 @@ namespace KeePass.Forms
 			PwDatabase pwDb = m_docMgr.ActiveDatabase;
 			PwEntry pwe = new PwEntry(true, true);
 			pwe.Strings.Set(PwDefs.UserNameField, new ProtectedString(
-				pwDb.MemoryProtection.ProtectUserName,
-				pwDb.DefaultUserName));
+				pwDb.MemoryProtection.ProtectUserName, pwDb.DefaultUserName));
 
-			ProtectedString psAutoGen = new ProtectedString(
-				pwDb.MemoryProtection.ProtectPassword);
-			PwGenerator.Generate(psAutoGen, Program.Config.PasswordGenerator.AutoGeneratedPasswordsProfile,
+			ProtectedString psAutoGen;
+			PwGenerator.Generate(out psAutoGen, Program.Config.PasswordGenerator.AutoGeneratedPasswordsProfile,
 				null, Program.PwGeneratorPool);
+			psAutoGen = psAutoGen.WithProtection(pwDb.MemoryProtection.ProtectPassword);
 			pwe.Strings.Set(PwDefs.PasswordField, psAutoGen);
 
 			int nExpireDays = Program.Config.Defaults.NewEntryExpiresInDays;
@@ -806,9 +841,7 @@ namespace KeePass.Forms
 
 				PwObjectList<PwEntry> vSelect = new PwObjectList<PwEntry>();
 				vSelect.Add(pwe);
-				SelectEntries(vSelect, true);
-				ListViewItem lviFocus = GuiFindEntry(pwe.Uuid);
-				if(lviFocus != null) m_lvEntries.FocusedItem = lviFocus;
+				SelectEntries(vSelect, true, true);
 
 				EnsureVisibleEntry(pwe.Uuid);
 			}
@@ -823,27 +856,43 @@ namespace KeePass.Forms
 
 		private void OnEntryDuplicate(object sender, EventArgs e)
 		{
+			PwDatabase pd = m_docMgr.ActiveDatabase;
 			PwGroup pgSelected = GetSelectedGroup();
 
 			PwEntry[] vSelected = GetSelectedEntries();
 			if((vSelected == null) || (vSelected.Length == 0)) return;
 
+			DuplicationForm dlg = new DuplicationForm();
+			if(UIUtil.ShowDialogAndDestroy(dlg) != DialogResult.OK) return;
+
 			PwObjectList<PwEntry> vNewEntries = new PwObjectList<PwEntry>();
 			foreach(PwEntry pe in vSelected)
 			{
 				PwEntry peNew = pe.CloneDeep();
 				peNew.SetUuid(new PwUuid(true), true); // Create new UUID
 
-				ProtectedString psTitle = peNew.Strings.Get(PwDefs.TitleField);
-				if(psTitle != null)
+				if(dlg.AppendCopyToTitles && (pd != null))
+				{
+					string strTitle = peNew.Strings.ReadSafe(PwDefs.TitleField);
 					peNew.Strings.Set(PwDefs.TitleField, new ProtectedString(
-						psTitle.IsProtected, psTitle.ReadString() + " - " +
+						pd.MemoryProtection.ProtectTitle, strTitle + " - " +
 						KPRes.CopyOfItem));
+				}
+
+				if(dlg.ReplaceDataByFieldRefs && (pd != null))
+				{
+					string strUser = @"{REF:U@I:" + pe.Uuid.ToHexString() + @"}";
+					peNew.Strings.Set(PwDefs.UserNameField, new ProtectedString(
+						pd.MemoryProtection.ProtectUserName, strUser));
+
+					string strPw = @"{REF:P@I:" + pe.Uuid.ToHexString() + @"}";
+					peNew.Strings.Set(PwDefs.PasswordField, new ProtectedString(
+						pd.MemoryProtection.ProtectPassword, strPw));
+				}
 
 				Debug.Assert(pe.ParentGroup == peNew.ParentGroup);
 				PwGroup pg = (pe.ParentGroup ?? pgSelected);
-				if((pg == null) && (m_docMgr.ActiveDocument != null))
-					pg = m_docMgr.ActiveDatabase.RootGroup;
+				if((pg == null) && (pd != null)) pg = pd.RootGroup;
 				if(pg == null) continue;
 
 				pg.AddEntry(peNew, true, true);
@@ -851,9 +900,9 @@ namespace KeePass.Forms
 			}
 
 			AddEntriesToList(vNewEntries);
-			SelectEntries(vNewEntries, true);
+			SelectEntries(vNewEntries, true, false);
 
-			if((m_lvEntries.ShowGroups == false) && (m_lvEntries.Items.Count >= 1))
+			if(!m_lvEntries.ShowGroups && (m_lvEntries.Items.Count >= 1))
 				m_lvEntries.EnsureVisible(m_lvEntries.Items.Count - 1);
 			else EnsureVisibleSelected(true);
 
@@ -892,7 +941,7 @@ namespace KeePass.Forms
 						SaveWindowPositionAndSize();
 
 						e.Cancel = true;
-						this.WindowState = FormWindowState.Minimized;
+						UIUtil.SetWindowState(this, FormWindowState.Minimized);
 						return;
 					}
 				}
@@ -905,6 +954,8 @@ namespace KeePass.Forms
 				return;
 			}
 
+			GlobalWindowManager.CloseAllWindows();
+
 			CleanUpEx(); // Saves configuration and cleans up all resources
 
 			if(m_bRestart) WinUtil.Restart();
@@ -1019,14 +1070,21 @@ namespace KeePass.Forms
 				{
 					if(lvs == lvHit.SubItem)
 					{
-						PerformDefaultAction(sender, e, lvi.Tag as PwEntry, i);
+						PwListItem pli = (lvi.Tag as PwListItem);
+						if(pli != null) PerformDefaultAction(sender, e, pli.Entry, i);
+						else { Debug.Assert(false); }
 						break;
 					}
 
 					++i;
 				}
 			}
-			else PerformDefaultAction(sender, e, lvi.Tag as PwEntry, 0);
+			else
+			{
+				PwListItem pli = (lvi.Tag as PwListItem);
+				if(pli != null) PerformDefaultAction(sender, e, pli.Entry, 0);
+				else { Debug.Assert(false); }
+			}
 		}
 
 		private void OnQuickFindSelectedIndexChanged(object sender, EventArgs e)
@@ -1035,15 +1093,30 @@ namespace KeePass.Forms
 			m_bBlockQuickFind = true;
 
 			string strSearch = m_tbQuickFind.Text; // Text, not selected index!
+
+			lock(m_objQuickFindSync)
+			{
+				int iNow = Environment.TickCount;
+				if(((iNow - m_iLastQuickFindTicks) <= 1000) &&
+					(strSearch == m_strLastQuickSearch))
+				{
+					m_bBlockQuickFind = false;
+					return;
+				}
+
+				m_iLastQuickFindTicks = iNow;
+				m_strLastQuickSearch = strSearch;
+			}
+
 			string strGroupName = KPRes.SearchGroupName + " (\"" + strSearch + "\" ";
 			strGroupName += KPRes.SearchResultsInSeparator + " ";
 			strGroupName += m_docMgr.ActiveDatabase.RootGroup.Name + ")";
 
-			PerformQuickFind(strSearch, strGroupName, false);
+			// PerformQuickFind(strSearch, strGroupName, false);
 
 			// Lookup in combobox for the current search
 			int nExistsAlready = -1;
-			for(int i = 0; i < m_tbQuickFind.Items.Count; i++)
+			for(int i = 0; i < m_tbQuickFind.Items.Count; ++i)
 			{
 				string strItemText = (string)m_tbQuickFind.Items[i];
 				if(strItemText.Equals(strSearch, StringComparison.InvariantCultureIgnoreCase))
@@ -1065,6 +1138,11 @@ namespace KeePass.Forms
 			m_tbQuickFind.SelectedIndex = 0;
 			m_tbQuickFind.Select(0, strSearch.Length);
 
+			// Asynchronous invokation allows to cleanly process
+			// an Enter keypress before blocking the UI
+			BeginInvoke(new PerformQuickFindDelegate(PerformQuickFind),
+				strSearch, strGroupName, false);
+
 			m_bBlockQuickFind = false;
 		}
 
@@ -1131,8 +1209,9 @@ namespace KeePass.Forms
 		private void OnPwListItemDrag(object sender, ItemDragEventArgs e)
 		{
 			if(e.Item == null) return;
-			PwEntry pe = (((ListViewItem)e.Item).Tag as PwEntry);
-			if(pe == null) { Debug.Assert(false); return; }
+			PwListItem pli = (((ListViewItem)e.Item).Tag as PwListItem);
+			if(pli == null) { Debug.Assert(false); return; }
+			PwEntry pe = pli.Entry;
 
 			ListViewHitTestInfo lvHit = m_lvEntries.HitTest(m_ptLastEntriesMouseClick);
 			ListViewItem lvi = lvHit.Item;
@@ -1149,7 +1228,8 @@ namespace KeePass.Forms
 				{
 					if(lvs == lvHit.SubItem)
 					{
-						strText = GetEntryFieldEx(pe, i, false);
+						bool bDummy;
+						strText = GetEntryFieldEx(pe, i, false, out bDummy);
 						break;
 					}
 
@@ -1159,8 +1239,9 @@ namespace KeePass.Forms
 
 			m_pgActiveAtDragStart = GetSelectedGroup();
 
-			PwDatabase pd = ((m_docMgr.ActiveDocument != null) ? m_docMgr.ActiveDatabase : null);
-			string strToTransfer = SprEngine.Compile(strText, false, pe, pd, false, false);
+			PwDatabase pd = m_docMgr.SafeFindContainerOf(pe);
+			string strToTransfer = SprEngine.Compile(strText, new SprContext(
+				pe, pd, SprCompileFlags.All));
 
 			m_bDraggingEntries = true;
 			this.DoDragDrop(strToTransfer, DragDropEffects.Copy | DragDropEffects.Move);
@@ -1315,7 +1396,7 @@ namespace KeePass.Forms
 			TreeNode tn = m_tvGroups.SelectedNode;
 			PwGroup pgParent;
 
-			if(tn != null) pgParent = tn.Tag as PwGroup;
+			if(tn != null) pgParent = (tn.Tag as PwGroup);
 			else pgParent = m_docMgr.ActiveDatabase.RootGroup;
 
 			if(pgParent == null) { Debug.Assert(false); return; }
@@ -1419,6 +1500,10 @@ namespace KeePass.Forms
 				if((m_fwsLast == FormWindowState.Minimized) && IsFileLocked(null) &&
 					!UIIsAutoUnlockBlocked())
 					OnFileLock(sender, e); // Unlock
+
+				if((m_fwsLast == FormWindowState.Minimized) && !IsFileLocked(null) &&
+					Program.Config.MainWindow.FocusQuickFindOnRestore)
+					ResetDefaultFocus(null);
 			}
 
 			m_fwsLast = ws;
@@ -1426,10 +1511,13 @@ namespace KeePass.Forms
 
 		private void OnTrayTray(object sender, EventArgs e)
 		{
+			if(!IsCommandTypeInvokable(null, AppCommandType.Window)) return;
+
 			if((this.WindowState == FormWindowState.Minimized) && !IsTrayed())
 			{
-				if(Program.Config.MainWindow.Maximized) this.WindowState = FormWindowState.Maximized;
-				else this.WindowState = FormWindowState.Normal;
+				if(Program.Config.MainWindow.Maximized)
+					UIUtil.SetWindowState(this, FormWindowState.Maximized);
+				else UIUtil.SetWindowState(this, FormWindowState.Normal);
 				return;
 			}
 
@@ -1514,7 +1602,8 @@ namespace KeePass.Forms
 			PwEntry[] v = GetSelectedEntries();
 			Debug.Assert(v != null); if(v == null) return;
 
-			if(ClipboardUtil.CopyAndMinimize(UrlsToString(v), true, this, null, null))
+			if(ClipboardUtil.CopyAndMinimize(UrlsToString(v, true), true,
+				this, null, null))
 				StartClipboardCountdown();
 		}
 
@@ -1586,7 +1675,11 @@ namespace KeePass.Forms
 					case Keys.C: OnEntryCopyPassword(sender, e); break;
 					case Keys.B: OnEntryCopyUserName(sender, e); break;
 					// case Keys.E: OnEntryEdit(sender, e); break;
-					case Keys.U: PerformDefaultUrlAction(null, false); break;
+					case Keys.U:
+						// PerformDefaultUrlAction(null, false);
+						if(e.Shift) OnEntryCopyURL(sender, e);
+						else OnEntryOpenUrl(sender, e);
+						break;
 					case Keys.V: OnEntryPerformAutoType(sender, e); break;
 					default: bUnhandled = true; break;
 				}
@@ -1702,9 +1795,10 @@ namespace KeePass.Forms
 
 					byte[] pbAdditionalEntropy = EntropyForm.CollectEntropyIfEnabled(
 						pgf.SelectedProfile);
-					ProtectedString psNew = new ProtectedString(pwDb.MemoryProtection.ProtectPassword);
-					PwGenerator.Generate(psNew, pgf.SelectedProfile, pbAdditionalEntropy,
-						Program.PwGeneratorPool);
+					ProtectedString psNew;
+					PwGenerator.Generate(out psNew, pgf.SelectedProfile,
+						pbAdditionalEntropy, Program.PwGeneratorPool);
+					psNew = psNew.WithProtection(pwDb.MemoryProtection.ProtectPassword);
 					pe.Strings.Set(PwDefs.PasswordField, psNew);
 
 					UpdateUI(false, null, false, null, true, null, true);
@@ -1723,7 +1817,7 @@ namespace KeePass.Forms
 
 		private void OnToolsShowExpired(object sender, EventArgs e)
 		{
-			ShowExpiredEntries(false, 0);
+			ShowExpiredEntries(false, true, false);
 		}
 
 		private void OnToolsTanWizard(object sender, EventArgs e)
@@ -1757,21 +1851,52 @@ namespace KeePass.Forms
 
 		private void OnEntryViewLinkClicked(object sender, LinkClickedEventArgs e)
 		{
+			string strLink = e.LinkText;
+			if(string.IsNullOrEmpty(strLink)) { Debug.Assert(false); return; }
+
 			PwEntry pe = GetSelectedEntry(false);
-			ProtectedBinary pb = ((pe != null) ? pe.Binaries.Get(e.LinkText) : null);
+			ProtectedBinary pb = ((pe != null) ? pe.Binaries.Get(strLink) : null);
+
+			string strEntryUrl = string.Empty;
+			if(pe != null)
+				strEntryUrl = SprEngine.Compile(pe.Strings.ReadSafe(PwDefs.UrlField),
+					GetEntryListSprContext(pe, m_docMgr.SafeFindContainerOf(pe)));
 
 			if((pe != null) && (pe.ParentGroup != null) &&
-				(pe.ParentGroup.Name == e.LinkText))
+				(pe.ParentGroup.Name == strLink))
 			{
 				UpdateUI(false, null, true, pe.ParentGroup, true, null, false);
 				EnsureVisibleSelected(false);
 				ResetDefaultFocus(m_lvEntries);
 			}
-			else if((pe != null) && (pe.Strings.ReadSafe(PwDefs.UrlField) == e.LinkText))
+			else if(strEntryUrl == strLink)
 				PerformDefaultUrlAction(null, false);
 			else if(pb != null)
-				ExecuteBinaryEditView(e.LinkText, pb);
-			else WinUtil.OpenUrl(e.LinkText, pe);
+				ExecuteBinaryEditView(strLink, pb);
+			else if(strLink.StartsWith(SprEngine.StrRefStart, StrUtil.CaseIgnoreCmp) &&
+				strLink.EndsWith(SprEngine.StrRefEnd, StrUtil.CaseIgnoreCmp))
+			{
+				// If multiple references are amalgamated, only use first one
+				string strFirstRef = strLink;
+				int iEnd = strLink.IndexOf(SprEngine.StrRefEnd, StrUtil.CaseIgnoreCmp);
+				if(iEnd != (strLink.Length - SprEngine.StrRefEnd.Length))
+					strFirstRef = strLink.Substring(0, iEnd + 1);
+
+				char chScan, chWanted;
+				PwEntry peRef = SprEngine.FindRefTarget(strFirstRef, GetEntryListSprContext(
+					pe, m_docMgr.SafeFindContainerOf(pe)), out chScan, out chWanted);
+				if(peRef != null)
+				{
+					UpdateUI(false, null, true, peRef.ParentGroup, true, null,
+						false, m_lvEntries);
+					PwObjectList<PwEntry> lSel = new PwObjectList<PwEntry>();
+					lSel.Add(peRef);
+					SelectEntries(lSel, true, true);
+					EnsureVisibleSelected(false);
+					ShowEntryDetails(peRef);
+				}
+			}
+			else WinUtil.OpenUrl(strLink, pe);
 		}
 
 		private void OnEntryClipCopy(object sender, EventArgs e)
@@ -1961,9 +2086,10 @@ namespace KeePass.Forms
 						PwEntry pe = new PwEntry(true, true);
 						pg.AddEntry(pe, true);
 
-						ProtectedString psNew = new ProtectedString(pwDb.MemoryProtection.ProtectPassword);
-						PwGenerator.Generate(psNew, pgf.SelectedProfile,
+						ProtectedString psNew;
+						PwGenerator.Generate(out psNew, pgf.SelectedProfile,
 							pbAdditionalEntropy, Program.PwGeneratorPool);
+						psNew = psNew.WithProtection(pwDb.MemoryProtection.ProtectPassword);
 						pe.Strings.Set(PwDefs.PasswordField, psNew);
 					}
 
@@ -2158,11 +2284,12 @@ namespace KeePass.Forms
 
 		private void OnCtxTrayOpening(object sender, CancelEventArgs e)
 		{
-			UpdateUIState(false);
+			UpdateTrayState();
 		}
 
 		private void OnTrayLock(object sender, EventArgs e)
 		{
+			if(!IsCommandTypeInvokable(null, AppCommandType.Lock)) return;
 			OnFileLock(sender, e);
 		}
 
@@ -2316,16 +2443,17 @@ namespace KeePass.Forms
 			PwDatabase pd = m_docMgr.ActiveDatabase;
 			if((pd == null) || !pd.IsOpen) { Debug.Assert(false); return; }
 
-			StatusProgressForm dlg = new StatusProgressForm();
-			dlg.InitEx(null, true, false, this);
-			dlg.Show();
-			dlg.StartLogging(KPRes.Delete + "...", false);
+			Form fOptDialog;
+			IStatusLogger sl = StatusUtil.CreateStatusDialog(this, out fOptDialog,
+				null, KPRes.Delete + "...", true, false);
+			if(fOptDialog != null) RedirectActivationPush(fOptDialog);
+			UIBlockInteraction(true);
 
-			uint uDeleted = pd.DeleteDuplicateEntries(dlg);
+			uint uDeleted = pd.DeleteDuplicateEntries(sl);
 
-			dlg.EndLogging();
-			dlg.Close();
-			UIUtil.DestroyForm(dlg);
+			if(fOptDialog != null) RedirectActivationPop();
+			UIBlockInteraction(false);
+			sl.EndLogging();
 
 			UpdateUI(false, null, false, null, (uDeleted > 0), null, (uDeleted > 0));
 			SetObjectsDeletedStatus(uDeleted, true);
@@ -2347,8 +2475,22 @@ namespace KeePass.Forms
 			if((pd == null) || !pd.IsOpen) { Debug.Assert(false); return; }
 
 			uint uDeleted = pd.DeleteUnusedCustomIcons();
-			UpdateUIState(uDeleted > 0);
+			UpdateUI(false, null, (uDeleted > 0), null, (uDeleted > 0),
+				null, (uDeleted > 0));
 			SetObjectsDeletedStatus(uDeleted, true);
 		}
+
+		private void OnTrayExit(object sender, EventArgs e)
+		{
+			if(!IsCommandTypeInvokable(null, AppCommandType.Window)) return;
+			OnFileExit(sender, e);
+		}
+
+		private void OnTrayOptions(object sender, EventArgs e)
+		{
+			if(!IsCommandTypeInvokable(null, AppCommandType.Window)) return;
+			EnsureVisibleForegroundWindow(false, false); // Parent is main window, not tray
+			OnToolsOptions(sender, e);
+		}
 	}
 }
