@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2011 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2012 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@ using KeePass.UI;
 using KeePass.Resources;
 
 using KeePassLib;
+using KeePassLib.Collections;
 using KeePassLib.Security;
 using KeePassLib.Delegates;
 using KeePassLib.Resources;
@@ -147,7 +148,7 @@ namespace KeePass.Forms
 
 			if(m_bPrintMode)
 			{
-				try { m_wbMain.Print(); } // Throws in Mono 1.2.6+
+				try { m_wbMain.ShowPrintDialog(); } // Throws in Mono 1.2.6+
 				catch(NotImplementedException)
 				{
 					MessageService.ShowWarning(KLRes.FrameworkNotImplExcp);
@@ -351,14 +352,17 @@ namespace KeePass.Forms
 
 					if(bAutoType)
 					{
-						foreach(KeyValuePair<string, string> kvp in pe.AutoType.WindowSequencePairs)
-							WriteDetailsLine(sb, KPRes.AutoType, kvp.Key + ": " + kvp.Value, bSmallMono, bMonoPasswords, strFontInit, strFontExit);
+						foreach(AutoTypeAssociation a in pe.AutoType.Associations)
+							WriteDetailsLine(sb, KPRes.AutoType, a.WindowName +
+								": " + a.Sequence, bSmallMono, bMonoPasswords,
+								strFontInit, strFontExit);
 					}
 
 					foreach(KeyValuePair<string, ProtectedString> kvp in pe.Strings)
 					{
 						if(bCustomStrings && !PwDefs.IsStandardField(kvp.Key))
-							WriteDetailsLine(sb, kvp, bSmallMono, bMonoPasswords, strFontInit, strFontExit);
+							WriteDetailsLine(sb, kvp, bSmallMono, bMonoPasswords,
+								strFontInit, strFontExit);
 					}
 
 					sb.AppendLine(@"<tr><td>&nbsp;</td></tr>");
