@@ -1,6 +1,6 @@
 ﻿/*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2011 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2012 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -48,6 +48,30 @@ namespace KeePass.Forms
 			get { return m_bCancelled; }
 		}
 
+		public static StatusProgressForm ConstructEx(string strTitle,
+			bool bCanCancel, bool bMarqueeProgress, Form fOwner,
+			string strInitialOp)
+		{
+			StatusProgressForm dlg = new StatusProgressForm();
+			dlg.InitEx(strTitle, bCanCancel, bMarqueeProgress, fOwner);
+
+			if(fOwner != null) dlg.Show(fOwner);
+			else dlg.Show();
+
+			dlg.StartLogging(strInitialOp, false);
+
+			return dlg;
+		}
+
+		public static void DestroyEx(StatusProgressForm dlg)
+		{
+			if(dlg == null) { Debug.Assert(false); return; }
+
+			dlg.EndLogging();
+			dlg.Close();
+			UIUtil.DestroyForm(dlg);
+		}
+
 		public void InitEx(string strTitle, bool bCanCancel, bool bMarqueeProgress,
 			Form fOwner)
 		{
@@ -106,7 +130,8 @@ namespace KeePass.Forms
 
 			if(strText != null) m_lblTotal.Text = strText;
 
-			if((nPercent >= 0) && (nPercent <= 100)) m_pbTotal.Value = nPercent;
+			if((nPercent >= 0) && (nPercent <= 100))
+				m_pbTotal.Value = nPercent; // .NET compares with cached value
 		}
 
 		private bool SetProgressGlobal(string strText, int nPercent)
