@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2011 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2012 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -87,7 +87,7 @@ namespace KeePass.UI
 			try
 			{
 				NativeMethods.CHARFORMAT2 cf = new NativeMethods.CHARFORMAT2();
-				cf.cbSize = (UInt32)Marshal.SizeOf(cf);
+				cf.cbSize = (uint)Marshal.SizeOf(cf);
 
 				cf.dwMask = NativeMethods.CFM_LINK;
 				cf.dwEffects = NativeMethods.CFE_LINK;
@@ -104,6 +104,35 @@ namespace KeePass.UI
 			catch(Exception) { Debug.Assert(false); }
 		}
 
+		private static NativeMethods.CHARFORMAT2 RtfGetCharFormat(RichTextBox rtb)
+		{
+			NativeMethods.CHARFORMAT2 cf = new NativeMethods.CHARFORMAT2();
+			cf.cbSize = (uint)Marshal.SizeOf(cf);
+
+			try
+			{
+				IntPtr wParam = (IntPtr)NativeMethods.SCF_SELECTION;
+				IntPtr lParam = Marshal.AllocCoTaskMem(Marshal.SizeOf(cf));
+				Marshal.StructureToPtr(cf, lParam, false);
+
+				NativeMethods.SendMessage(rtb.Handle,
+					NativeMethods.EM_GETCHARFORMAT, wParam, lParam);
+
+				cf = (NativeMethods.CHARFORMAT2)Marshal.PtrToStructure(lParam,
+					typeof(NativeMethods.CHARFORMAT2));
+				Marshal.FreeCoTaskMem(lParam);
+			}
+			catch(Exception) { Debug.Assert(false); }
+
+			return cf;
+		}
+
+		public static bool RtfIsFirstCharLink(RichTextBox rtb)
+		{
+			NativeMethods.CHARFORMAT2 cf = RtfGetCharFormat(rtb);
+			return ((cf.dwEffects & NativeMethods.CFE_LINK) != 0);
+		}
+
 		public static void RtfLinkifyExtUrls(RichTextBox richTextBox, bool bResetSelection)
 		{
 			const string strProto = "cmd://";
@@ -151,6 +180,32 @@ namespace KeePass.UI
 			catch(Exception) { Debug.Assert(false); }
 		}
 
+		public static void RtfLinkifyReferences(RichTextBox rtb,
+			bool bResetTempSelection)
+		{
+			try
+			{
+				string str = rtb.Text;
+
+				int iOffset = 0;
+				while(true)
+				{
+					int iStart = str.IndexOf(SprEngine.StrRefStart, iOffset,
+						StrUtil.CaseIgnoreCmp);
+					if(iStart < 0) break;
+					int iEnd = str.IndexOf(SprEngine.StrRefEnd, iStart + 1,
+						StrUtil.CaseIgnoreCmp);
+					if(iEnd <= iStart) break;
+
+					string strRef = str.Substring(iStart, iEnd - iStart + 1);
+					RtfLinkifyText(rtb, strRef, bResetTempSelection);
+
+					iOffset = iStart + 1;
+				}
+			}
+			catch(Exception) { Debug.Assert(false); }
+		}
+
 		public static Image CreateColorBitmap24(int nWidth, int nHeight, Color color)
 		{
 			Bitmap bmp = new Bitmap(nWidth, nHeight, PixelFormat.Format24bppRgb);
@@ -420,7 +475,6 @@ namespace KeePass.UI
 				if(pe.ParentGroup != null)
 				{
 					string strGroup = pe.ParentGroup.GetFullPath();
-
 					if(strGroup != lvg.Header)
 					{
 						lvg = new ListViewGroup(strGroup, HorizontalAlignment.Left);
@@ -483,6 +537,125 @@ namespace KeePass.UI
 			lv.EndUpdate();
 		}
 
+		/// <summary>
+		/// Fill a <c>ListView</c> with password entries.
+		/// </summary>
+		public static void CreateEntryList(ListView lv, List<AutoTypeCtx> lCtxs,
+			AceAutoTypeCtxFlags f, ImageList ilIcons)
+		{
+			if(lv == null) throw new ArgumentNullException("lv");
+			if(lCtxs == null) throw new ArgumentNullException("lCtxs");
+
+			lv.BeginUpdate();
+
+			lv.Items.Clear();
+			lv.Columns.Clear();
+			lv.ShowGroups = true;
+			lv.SmallImageList = ilIcons;
+
+			Debug.Assert((f & AceAutoTypeCtxFlags.ColTitle) != AceAutoTypeCtxFlags.None);
+			f |= AceAutoTypeCtxFlags.ColTitle; // Enforce title
+
+			lv.Columns.Add(KPRes.Title);
+			if((f & AceAutoTypeCtxFlags.ColUserName) != AceAutoTypeCtxFlags.None)
+				lv.Columns.Add(KPRes.UserName);
+			if((f & AceAutoTypeCtxFlags.ColPassword) != AceAutoTypeCtxFlags.None)
+				lv.Columns.Add(KPRes.Password);
+			if((f & AceAutoTypeCtxFlags.ColUrl) != AceAutoTypeCtxFlags.None)
+				lv.Columns.Add(KPRes.Url);
+			if((f & AceAutoTypeCtxFlags.ColNotes) != AceAutoTypeCtxFlags.None)
+				lv.Columns.Add(KPRes.Notes);
+			if((f & AceAutoTypeCtxFlags.ColSequence) != AceAutoTypeCtxFlags.None)
+				lv.Columns.Add(KPRes.Sequence);
+
+			ListViewGroup lvg = new ListViewGroup(Guid.NewGuid().ToString());
+			DateTime dtNow = DateTime.Now;
+			bool bFirstEntry = true;
+
+			foreach(AutoTypeCtx ctx in lCtxs)
+			{
+				if(ctx == null) { Debug.Assert(false); continue; }
+				PwEntry pe = ctx.Entry;
+				if(pe == null) { Debug.Assert(false); continue; }
+				PwDatabase pd = ctx.Database;
+				if(pd == null) { Debug.Assert(false); continue; }
+
+				if(pe.ParentGroup != null)
+				{
+					string strGroup = pe.ParentGroup.GetFullPath(" - ", true);
+					if(strGroup != lvg.Header)
+					{
+						lvg = new ListViewGroup(strGroup, HorizontalAlignment.Left);
+						lv.Groups.Add(lvg);
+					}
+				}
+
+				SprContext sprCtx = new SprContext(pe, pd, SprCompileFlags.Deref);
+				sprCtx.ForcePlainTextPasswords = false;
+
+				ListViewItem lvi = new ListViewItem(SprEngine.Compile(
+					pe.Strings.ReadSafe(PwDefs.TitleField), sprCtx));
+
+				if(pe.Expires && (pe.ExpiryTime <= dtNow))
+					lvi.ImageIndex = (int)PwIcon.Expired;
+				else if(pe.CustomIconUuid == PwUuid.Zero)
+					lvi.ImageIndex = (int)pe.IconId;
+				else
+				{
+					int nInx = pd.GetCustomIconIndex(pe.CustomIconUuid);
+					if(nInx > -1)
+					{
+						ilIcons.Images.Add(new Bitmap(pd.GetCustomIcon(
+							pe.CustomIconUuid)));
+						lvi.ImageIndex = ilIcons.Images.Count - 1;
+					}
+					else { Debug.Assert(false); lvi.ImageIndex = (int)pe.IconId; }
+				}
+
+				if((f & AceAutoTypeCtxFlags.ColUserName) != AceAutoTypeCtxFlags.None)
+					lvi.SubItems.Add(SprEngine.Compile(pe.Strings.ReadSafe(
+						PwDefs.UserNameField), sprCtx));
+				if((f & AceAutoTypeCtxFlags.ColPassword) != AceAutoTypeCtxFlags.None)
+					lvi.SubItems.Add(SprEngine.Compile(pe.Strings.ReadSafe(
+						PwDefs.PasswordField), sprCtx));
+				if((f & AceAutoTypeCtxFlags.ColUrl) != AceAutoTypeCtxFlags.None)
+					lvi.SubItems.Add(SprEngine.Compile(pe.Strings.ReadSafe(
+						PwDefs.UrlField), sprCtx));
+				if((f & AceAutoTypeCtxFlags.ColNotes) != AceAutoTypeCtxFlags.None)
+					lvi.SubItems.Add(StrUtil.MultiToSingleLine(SprEngine.Compile(
+						pe.Strings.ReadSafe(PwDefs.NotesField), sprCtx)));
+				if((f & AceAutoTypeCtxFlags.ColSequence) != AceAutoTypeCtxFlags.None)
+					lvi.SubItems.Add(ctx.Sequence);
+				Debug.Assert(lvi.SubItems.Count == lv.Columns.Count);
+
+				if(!pe.ForegroundColor.IsEmpty)
+					lvi.ForeColor = pe.ForegroundColor;
+				if(!pe.BackgroundColor.IsEmpty)
+					lvi.BackColor = pe.BackgroundColor;
+
+				lvi.Tag = ctx;
+
+				lv.Items.Add(lvi);
+				lvg.Items.Add(lvi);
+
+				if(bFirstEntry)
+				{
+					lvi.Selected = true;
+					lvi.Focused = true;
+
+					bFirstEntry = false;
+				}
+			}
+
+			lv.EndUpdate();
+
+			int nColWidth = lv.ClientRectangle.Width / lv.Columns.Count;
+			foreach(ColumnHeader ch in lv.Columns)
+			{
+				ch.Width = nColWidth;
+			}
+		}
+
 		public static int GetVScrollBarWidth()
 		{
 			try { return SystemInformation.VerticalScrollBarWidth; }
@@ -682,15 +855,46 @@ namespace KeePass.UI
 		{
 			if((tn == null) || (pg == null)) { Debug.Assert(false); return; }
 
-			if(pg.Notes.Length > 0)
-			{
-				string str = pg.Name + MessageService.NewParagraph + pg.Notes;
+			string str = GetPwGroupToolTip(pg);
+			if(str == null) return;
 
-				try { tn.ToolTipText = str; }
-				catch(Exception) { Debug.Assert(false); }
-			}
+			try { tn.ToolTipText = str; }
+			catch(Exception) { Debug.Assert(false); }
 		}
 
+		public static string GetPwGroupToolTip(PwGroup pg)
+		{
+			if(pg == null) { Debug.Assert(false); return null; }
+
+			StringBuilder sb = new StringBuilder();
+			sb.Append(pg.Name);
+
+			string strNotes = pg.Notes.Trim();
+			if(strNotes.Length > 0)
+			{
+				sb.Append(MessageService.NewParagraph);
+				sb.Append(strNotes);
+			}
+			else return null;
+
+			// uint uSubGroups, uEntries;
+			// pg.GetCounts(true, out uSubGroups, out uEntries);
+			// sb.Append(MessageService.NewParagraph);
+			// sb.Append(KPRes.Subgroups); sb.Append(": "); sb.Append(uSubGroups);
+			// sb.Append(MessageService.NewLine);
+			// sb.Append(KPRes.Entries); sb.Append(": "); sb.Append(uEntries);
+
+			return sb.ToString();
+		}
+
+		// public static string GetPwGroupToolTipTN(TreeNode tn)
+		// {
+		//	if(tn == null) { Debug.Assert(false); return null; }
+		//	PwGroup pg = (tn.Tag as PwGroup);
+		//	if(pg == null) { Debug.Assert(false); return null; }
+		//	return GetPwGroupToolTip(pg);
+		// }
+
 		public static Color LightenColor(Color clrBase, double dblFactor)
 		{
 			if((dblFactor <= 0.0) || (dblFactor > 1.0)) return clrBase;
@@ -807,7 +1011,6 @@ namespace KeePass.UI
 			if(pb == null) throw new ArgumentNullException("pb");
 
 			MemoryStream ms = new MemoryStream(pb, false);
-
 			try { return Image.FromStream(ms); }
 			catch(Exception)
 			{
@@ -816,14 +1019,15 @@ namespace KeePass.UI
 
 				throw;
 			}
+			finally { ms.Close(); }
 		}
 
 		private static Image TryLoadIco(byte[] pb)
 		{
 			MemoryStream ms = new MemoryStream(pb, false);
-
 			try { return (new Icon(ms)).ToBitmap(); }
 			catch(Exception) { }
+			finally { ms.Close(); }
 
 			return null;
 		}
@@ -851,13 +1055,33 @@ namespace KeePass.UI
 
 		public static void ConfigureTbButton(ToolStripItem tb, string strText,
 			string strTooltip)
+		{
+			ConfigureTbButton(tb, strText, strTooltip, null);
+		}
+
+		private static char[] m_vTbTrim = null;
+		public static void ConfigureTbButton(ToolStripItem tb, string strText,
+			string strTooltip, ToolStripMenuItem tsmiEquiv)
 		{
 			if(strText != null) tb.Text = strText;
 
-			if(strTooltip != null)
-				tb.ToolTipText = StrUtil.RemoveAccelerator(strTooltip);
-			else if(strText != null)
-				tb.ToolTipText = StrUtil.RemoveAccelerator(strText);
+			if(m_vTbTrim == null)
+				m_vTbTrim = new char[] { ' ', '\t', '\r', '\n', '.', '\u2026' };
+
+			string strTip = (strTooltip ?? strText);
+			if(strTip == null) return;
+
+			strTip = StrUtil.RemoveAccelerator(strTip);
+			strTip = strTip.Trim(m_vTbTrim);
+
+			if((tsmiEquiv != null) && (strTip.Length > 0))
+			{
+				string strShortcut = tsmiEquiv.ShortcutKeyDisplayString;
+				if(!string.IsNullOrEmpty(strShortcut))
+					strTip += " (" + strShortcut + ")";
+			}
+
+			tb.ToolTipText = strTip;
 		}
 
 		public static void CreateGroupList(PwGroup pgContainer, ComboBox cmb,
@@ -989,27 +1213,53 @@ namespace KeePass.UI
 
 			Color clrBg = lv.BackColor;
 
-			// Items in sorted groups don't have display-ordered indices
-			// (in contrast to a non-grouped sorted list)
-			if(lv.ShowGroups && (lv.ListViewItemSorter != null))
-				bAlternate = false;
-
-			for(int i = 0; i < lv.Items.Count; ++i)
+			if(!lv.ShowGroups || !bAlternate)
 			{
-				ListViewItem lvi = lv.Items[i];
-				Debug.Assert(lvi.Index == i);
-				Debug.Assert(lvi.UseItemStyleForSubItems);
-
-				if(!bAlternate)
+				for(int i = 0; i < lv.Items.Count; ++i)
 				{
-					if(ColorsEqual(lvi.BackColor, clrAlternate))
+					ListViewItem lvi = lv.Items[i];
+					Debug.Assert(lvi.Index == i);
+					Debug.Assert(lvi.UseItemStyleForSubItems);
+
+					if(!bAlternate)
+					{
+						if(ColorsEqual(lvi.BackColor, clrAlternate))
+							lvi.BackColor = clrBg;
+					}
+					else if(((i & 1) == 0) && ColorsEqual(lvi.BackColor, clrAlternate))
 						lvi.BackColor = clrBg;
+					else if(((i & 1) == 1) && ColorsEqual(lvi.BackColor, clrBg))
+						lvi.BackColor = clrAlternate;
 				}
-				else if(((i & 1) == 0) && ColorsEqual(lvi.BackColor, clrAlternate))
-					lvi.BackColor = clrBg;
-				else if(((i & 1) == 1) && ColorsEqual(lvi.BackColor, clrBg))
-					lvi.BackColor = clrAlternate;
 			}
+			else // Groups && alternating
+			{
+				foreach(ListViewGroup lvg in lv.Groups)
+				{
+					// Within the group the items are not in display order,
+					// but the order can be derived from the item indices
+					List<ListViewItem> lItems = new List<ListViewItem>();
+					foreach(ListViewItem lviEnum in lvg.Items)
+						lItems.Add(lviEnum);
+					lItems.Sort(UIUtil.LviCompareByIndex);
+
+					for(int i = 0; i < lItems.Count; ++i)
+					{
+						ListViewItem lvi = lItems[i];
+						Debug.Assert(lvi.UseItemStyleForSubItems);
+
+						if(((i & 1) == 0) && ColorsEqual(lvi.BackColor, clrAlternate))
+							lvi.BackColor = clrBg;
+						else if(((i & 1) == 1) && ColorsEqual(lvi.BackColor, clrBg))
+							lvi.BackColor = clrAlternate;
+					}
+				}
+			}
+		}
+
+		private static int LviCompareByIndex(ListViewItem a, ListViewItem b)
+		{
+			return a.Index.CompareTo(b.Index);
 		}
 
 		public static bool SetSortIcon(ListView lv, int iColumn, SortOrder so)
@@ -1275,6 +1525,30 @@ namespace KeePass.UI
 			else { Debug.Assert(false); }
 		}
 
+		public static string GetColumnWidths(ListView lv)
+		{
+			if(lv == null) { Debug.Assert(false); return string.Empty; }
+
+			int n = lv.Columns.Count;
+			int[] vSizes = new int[n];
+			for(int i = 0; i < n; ++i) vSizes[i] = lv.Columns[i].Width;
+
+			return StrUtil.SerializeIntArray(vSizes);
+		}
+
+		public static void SetColumnWidths(ListView lv, string strSizes)
+		{
+			if(string.IsNullOrEmpty(strSizes)) return; // No assert
+
+			int[] vSizes = StrUtil.DeserializeIntArray(strSizes);
+
+			int n = lv.Columns.Count;
+			Debug.Assert(n == vSizes.Length);
+
+			for(int i = 0; i < Math.Min(n, vSizes.Length); ++i)
+				lv.Columns[i].Width = vSizes[i];
+		}
+
 		public static void SetButtonImage(Button btn, Image img, bool b16To15)
 		{
 			if(btn == null) { Debug.Assert(false); return; }
@@ -1606,7 +1880,7 @@ namespace KeePass.UI
 				if(string.IsNullOrEmpty(strWav))
 					strWav = @"%SystemRoot%\Media\Windows User Account Control.wav";
 
-				strWav = SprEngine.Compile(strWav, false, null, null, false, false);
+				strWav = SprEngine.Compile(strWav, null);
 
 				if(!File.Exists(strWav)) throw new FileNotFoundException();
 
@@ -1663,5 +1937,86 @@ namespace KeePass.UI
 
 			return null;
 		}
+
+		/// <summary>
+		/// Set the state of a window. This is a workaround for
+		/// https://sourceforge.net/projects/keepass/forums/forum/329221/topic/4610118
+		/// </summary>
+		public static void SetWindowState(Form f, FormWindowState fws)
+		{
+			if(f == null) { Debug.Assert(false); return; }
+
+			f.WindowState = fws;
+
+			// If the window state change / resize handler changes
+			// the window state again, the property gets out of sync
+			// with the real state. Therefore, we now make sure that
+			// the property is synchronized with the actual window
+			// state.
+			try
+			{
+				// Get the value that .NET currently caches; note
+				// this isn't necessarily the real window state
+				// due to the bug
+				FormWindowState fwsCached = f.WindowState;
+
+				IntPtr hWnd = f.Handle;
+				if(hWnd == IntPtr.Zero) { Debug.Assert(false); return; }
+
+				// Get the real state using Windows API functions
+				bool bIsRealMin = NativeMethods.IsIconic(hWnd);
+				bool bIsRealMax = NativeMethods.IsZoomed(hWnd);
+
+				FormWindowState? fwsFix = null;
+				if(bIsRealMin && (fwsCached != FormWindowState.Minimized))
+					fwsFix = FormWindowState.Minimized;
+				else if(bIsRealMax && (fwsCached != FormWindowState.Maximized) &&
+					!bIsRealMin)
+					fwsFix = FormWindowState.Maximized;
+				else if(!bIsRealMin && !bIsRealMax &&
+					(fwsCached != FormWindowState.Normal))
+					fwsFix = FormWindowState.Normal;
+
+				if(fwsFix.HasValue)
+				{
+					// If the window is invisible, no state
+					// change / resize handlers are called
+					bool bVisible = f.Visible;
+					if(bVisible) f.Visible = false;
+					f.WindowState = fwsFix.Value;
+					if(bVisible) f.Visible = true;
+				}
+			}
+			catch(Exception) { Debug.Assert(KeePassLib.Native.NativeLib.IsUnix()); }
+		}
+
+		private static KeysConverter m_convKeys = null;
+		public static string GetKeysName(Keys k)
+		{
+			if(m_convKeys == null) m_convKeys = new KeysConverter();
+			return m_convKeys.ConvertToString(k);
+		}
+
+		/// <summary>
+		/// Assign shortcut keys to a menu item. This method uses
+		/// custom-translated display strings.
+		/// </summary>
+		public static void AssignShortcut(ToolStripMenuItem tsmi, Keys k)
+		{
+			if(tsmi == null) { Debug.Assert(false); return; }
+
+			tsmi.ShortcutKeys = k;
+
+			string str = string.Empty;
+			if((k & Keys.Control) != Keys.None)
+				str += KPRes.KeyboardKeyCtrl + "+";
+			if((k & Keys.Alt) != Keys.None)
+				str += KPRes.KeyboardKeyAlt + "+";
+			if((k & Keys.Shift) != Keys.None)
+				str += KPRes.KeyboardKeyShift + "+";
+			str += GetKeysName(k & Keys.KeyCode);
+
+			tsmi.ShortcutKeyDisplayString = str;
+		}
 	}
 }
