@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2011 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2012 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -105,11 +105,10 @@ namespace KeePass.Util
 			try
 			{
 				Image.FromStream(ms);
-
-				ms.Close();
 				return BinaryDataClass.Image;
 			}
-			catch(Exception) { ms.Close(); }
+			catch(Exception) { }
+			finally { ms.Close(); }
 
 			return BinaryDataClass.Unknown;
 		}
