@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2011 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2012 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@ using System.Threading;
 using KeePass.App;
 using KeePass.Ecas;
 using KeePass.Native;
+using KeePass.UI;
 using KeePass.Util;
 using KeePass.Util.Spr;
 
@@ -76,8 +77,9 @@ namespace KeePass.Util
 			if(bIsEntryInfo && !AppPolicy.Try(AppPolicyId.CopyToClipboard))
 				return false;
 
-			string strData = (bSprCompile ? SprEngine.Compile(strToCopy, false,
-				peEntryInfo, pwReferenceSource, false, false) : strToCopy);
+			string strData = (bSprCompile ? SprEngine.Compile(strToCopy,
+				new SprContext(peEntryInfo, pwReferenceSource,
+				SprCompileFlags.All)) : strToCopy);
 
 			try
 			{
@@ -232,7 +234,7 @@ namespace KeePass.Util
 						NativeMethods.LoseFocus(formContext);
 
 					if(Program.Config.MainWindow.MinimizeAfterClipboardCopy)
-						formContext.WindowState = FormWindowState.Minimized;
+						UIUtil.SetWindowState(formContext, FormWindowState.Minimized);
 				}
 
 				return true;
