@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2011 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2012 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -72,7 +72,6 @@ namespace KeePassLib
 			set
 			{
 				Debug.Assert(value != null); if(value == null) throw new ArgumentNullException("value");
-
 				m_uuid = value;
 			}
 		}
@@ -106,7 +105,6 @@ namespace KeePassLib
 			set
 			{
 				Debug.Assert(value != null); if(value == null) throw new ArgumentNullException("value");
-				
 				m_listStrings = value;
 			}
 		}
@@ -120,7 +118,6 @@ namespace KeePassLib
 			set
 			{
 				Debug.Assert(value != null); if(value == null) throw new ArgumentNullException("value");
-				
 				m_listBinaries = value;
 			}
 		}
@@ -134,7 +131,6 @@ namespace KeePassLib
 			set
 			{
 				Debug.Assert(value != null); if(value == null) throw new ArgumentNullException("value");
-				
 				m_listAutoType = value;
 			}
 		}
@@ -148,7 +144,6 @@ namespace KeePassLib
 			set
 			{
 				Debug.Assert(value != null); if(value == null) throw new ArgumentNullException("value");
-
 				m_listHistory = value;
 			}
 		}
@@ -173,7 +168,6 @@ namespace KeePassLib
 			set
 			{
 				Debug.Assert(value != null); if(value == null) throw new ArgumentNullException("value");
-
 				m_pwCustomIconID = value;
 			}
 		}
@@ -261,7 +255,6 @@ namespace KeePassLib
 			set
 			{
 				if(value == null) throw new ArgumentNullException("value");
-
 				m_strOverrideUrl = value;
 			}
 		}
@@ -275,7 +268,6 @@ namespace KeePassLib
 			set
 			{
 				if(value == null) throw new ArgumentNullException("value");
-
 				m_vTags = value;
 			}
 		}
@@ -378,39 +370,87 @@ namespace KeePassLib
 			return peNew;
 		}
 
+		private static PwCompareOptions BuildCmpOpt(bool bIgnoreParentGroup,
+			bool bIgnoreLastMod, bool bIgnoreLastAccess, bool bIgnoreHistory,
+			bool bIgnoreThisLastBackup)
+		{
+			PwCompareOptions pwOpt = PwCompareOptions.None;
+			if(bIgnoreParentGroup) pwOpt |= PwCompareOptions.IgnoreParentGroup;
+			if(bIgnoreLastMod) pwOpt |= PwCompareOptions.IgnoreLastMod;
+			if(bIgnoreLastAccess) pwOpt |= PwCompareOptions.IgnoreLastAccess;
+			if(bIgnoreHistory) pwOpt |= PwCompareOptions.IgnoreHistory;
+			if(bIgnoreThisLastBackup) pwOpt |= PwCompareOptions.IgnoreLastBackup;
+			return pwOpt;
+		}
+
+		[Obsolete]
 		public bool EqualsEntry(PwEntry pe, bool bIgnoreParentGroup, bool bIgnoreLastMod,
 			bool bIgnoreLastAccess, bool bIgnoreHistory, bool bIgnoreThisLastBackup)
+		{
+			return EqualsEntry(pe, BuildCmpOpt(bIgnoreParentGroup, bIgnoreLastMod,
+				bIgnoreLastAccess, bIgnoreHistory, bIgnoreThisLastBackup),
+				MemProtCmpMode.None);
+		}
+
+		[Obsolete]
+		public bool EqualsEntry(PwEntry pe, bool bIgnoreParentGroup, bool bIgnoreLastMod,
+			bool bIgnoreLastAccess, bool bIgnoreHistory, bool bIgnoreThisLastBackup,
+			MemProtCmpMode mpCmpStr)
+		{
+			return EqualsEntry(pe, BuildCmpOpt(bIgnoreParentGroup, bIgnoreLastMod,
+				bIgnoreLastAccess, bIgnoreHistory, bIgnoreThisLastBackup), mpCmpStr);
+		}
+
+		public bool EqualsEntry(PwEntry pe, PwCompareOptions pwOpt,
+			MemProtCmpMode mpCmpStr)
 		{
 			if(pe == null) { Debug.Assert(false); return false; }
 
+			bool bNeEqStd = ((pwOpt & PwCompareOptions.NullEmptyEquivStd) !=
+				PwCompareOptions.None);
+			bool bIgnoreLastAccess = ((pwOpt & PwCompareOptions.IgnoreLastAccess) !=
+				PwCompareOptions.None);
+			bool bIgnoreLastMod = ((pwOpt & PwCompareOptions.IgnoreLastMod) !=
+				PwCompareOptions.None);
+
 			if(!m_uuid.EqualsValue(pe.m_uuid)) return false;
-			if(!bIgnoreParentGroup)
+			if((pwOpt & PwCompareOptions.IgnoreParentGroup) == PwCompareOptions.None)
 			{
 				if(m_pParentGroup != pe.m_pParentGroup) return false;
 				if(!bIgnoreLastMod && (m_tParentGroupLastMod != pe.m_tParentGroupLastMod))
 					return false;
 			}
 
-			if(!m_listStrings.EqualsDictionary(pe.m_listStrings)) return false;
+			if(!m_listStrings.EqualsDictionary(pe.m_listStrings, pwOpt, mpCmpStr))
+				return false;
 			if(!m_listBinaries.EqualsDictionary(pe.m_listBinaries)) return false;
 
-			if(!m_listAutoType.EqualsConfig(pe.m_listAutoType)) return false;
+			if(!m_listAutoType.Equals(pe.m_listAutoType)) return false;
 
-			if(!bIgnoreHistory)
+			if((pwOpt & PwCompareOptions.IgnoreHistory) == PwCompareOptions.None)
 			{
-				if(!bIgnoreThisLastBackup && (m_listHistory.UCount != pe.m_listHistory.UCount))
+				bool bIgnoreLastBackup = ((pwOpt & PwCompareOptions.IgnoreLastBackup) !=
+					PwCompareOptions.None);
+
+				if(!bIgnoreLastBackup && (m_listHistory.UCount != pe.m_listHistory.UCount))
 					return false;
-				if(bIgnoreThisLastBackup && (m_listHistory.UCount == 0))
+				if(bIgnoreLastBackup && (m_listHistory.UCount == 0))
 				{
 					Debug.Assert(false);
 					return false;
 				}
-				if(bIgnoreThisLastBackup && ((m_listHistory.UCount - 1) != pe.m_listHistory.UCount))
+				if(bIgnoreLastBackup && ((m_listHistory.UCount - 1) != pe.m_listHistory.UCount))
 					return false;
+
+				PwCompareOptions cmpSub = PwCompareOptions.IgnoreParentGroup;
+				if(bNeEqStd) cmpSub |= PwCompareOptions.NullEmptyEquivStd;
+				if(bIgnoreLastMod) cmpSub |= PwCompareOptions.IgnoreLastMod;
+				if(bIgnoreLastAccess) cmpSub |= PwCompareOptions.IgnoreLastAccess;
+
 				for(uint uHist = 0; uHist < pe.m_listHistory.UCount; ++uHist)
 				{
 					if(!m_listHistory.GetAt(uHist).EqualsEntry(pe.m_listHistory.GetAt(
-						uHist), true, bIgnoreLastMod, bIgnoreLastAccess, false, false))
+						uHist), cmpSub, MemProtCmpMode.None))
 						return false;
 				}
 			}
@@ -589,10 +629,14 @@ namespace KeePassLib
 		{
 			if(peData == null) { Debug.Assert(false); return false; }
 
+			PwCompareOptions cmpOpt = (PwCompareOptions.IgnoreParentGroup |
+				PwCompareOptions.IgnoreHistory | PwCompareOptions.NullEmptyEquivStd);
+			if(bIgnoreLastMod) cmpOpt |= PwCompareOptions.IgnoreLastMod;
+			if(bIgnoreLastAccess) cmpOpt |= PwCompareOptions.IgnoreLastAccess;
+
 			foreach(PwEntry pe in m_listHistory)
 			{
-				if(pe.EqualsEntry(peData, true, bIgnoreLastMod, bIgnoreLastAccess,
-					true, false)) return true;
+				if(pe.EqualsEntry(peData, cmpOpt, MemProtCmpMode.None)) return true;
 			}
 
 			return false;
@@ -660,7 +704,7 @@ namespace KeePassLib
 
 		public bool GetAutoTypeEnabled()
 		{
-			if(m_listAutoType.Enabled == false) return false;
+			if(!m_listAutoType.Enabled) return false;
 
 			if(m_pParentGroup != null)
 				return m_pParentGroup.GetAutoTypeEnabledInherited();
@@ -717,10 +761,10 @@ namespace KeePassLib
 			}
 
 			uSize += (ulong)m_listAutoType.DefaultSequence.Length;
-			foreach(KeyValuePair<string, string> kvpCfg in m_listAutoType.WindowSequencePairs)
+			foreach(AutoTypeAssociation a in m_listAutoType.Associations)
 			{
-				uSize += (ulong)kvpCfg.Key.Length;
-				uSize += (ulong)kvpCfg.Value.Length;
+				uSize += (ulong)a.WindowName.Length;
+				uSize += (ulong)a.Sequence.Length;
 			}
 
 			foreach(PwEntry peHistory in m_listHistory)
