@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2011 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2012 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -44,12 +44,14 @@ namespace TrlUtil
 			List<KPFormCustomization> l = new List<KPFormCustomization>();
 
 			AddForm(l, new AboutForm());
+			AddForm(l, new AutoTypeCtxForm());
 			AddForm(l, new CharPickerForm());
 			AddForm(l, new ColumnsForm());
 			AddForm(l, new DatabaseOperationsForm());
 			AddForm(l, new DatabaseSettingsForm());
 			AddForm(l, new DataEditorForm());
 			AddForm(l, new DataViewerForm());
+			AddForm(l, new DuplicationForm());
 			AddForm(l, new EcasActionForm());
 			AddForm(l, new EcasConditionForm());
 			AddForm(l, new EcasEventForm());
@@ -86,6 +88,7 @@ namespace TrlUtil
 			AddForm(l, new StatusProgressForm());
 			AddForm(l, new TanWizardForm());
 			AddForm(l, new TextEncodingForm());
+			AddForm(l, new UpdateCheckForm());
 			AddForm(l, new UrlSchemeForm());
 			AddForm(l, new UrlSchemesForm());
 
@@ -134,6 +137,7 @@ namespace TrlUtil
 			else if(t == typeof(RichTextBox)) bAdd = true;
 			else if(t == typeof(KeePass.UI.CustomRichTextBoxEx)) bAdd = true;
 			else if(t == typeof(ComboBox)) bAdd = true;
+			else if(t == typeof(KeePass.UI.ImageComboBoxEx)) bAdd = true;
 			else if(t == typeof(Label)) bAdd = true;
 			else if(t == typeof(ListView)) bAdd = true;
 			else if(t == typeof(CustomListViewEx)) bAdd = true;
